package org.wwbota.identification;
import java.util.*;

/** A Description is a list of characters with corresponding criteria.
    It's like field-value pairs, except that fields ({@link Character}) and values ({@link Criterion}) have more information than just strings 
*/
public interface Description {
 /** Accessor to the Taxon (item) being described */
 Taxon getTaxon();
 /** Add a Character-Criterion pair */
 void addCriterion(Criterion c);
 /** Accessor to Criterion from Character.
  This is a helper function, since the getMap() accessor gives the same information, plus the ability to iterate on all criteria */
 Criterion getCriterion(Character c);
 /** @return a Map of {@link Criterion} objects; the key is the corresponding {@link Character} */
 Map getMap();
 /** @return the probability that this Description corresponds to the given reference.
  This is a helper function, since it gets the information by iterating on all criteria, and calling the same function on each. */
 double isCompatibleWith(Description reference);
}
