package org.wwbota.identification;
import java.util.*;

/** Computer-aided interactive Identification (determination) of a biological specimen, believed to belong to one of the taxa of the current {@link Taxonomy Taxonomy} .

This is the central interface of this API. It should allow to integrate a specific Identification algorithm, plus the Taxonomy on which it operates, in a GUI (Graphical User Interface) or other software.

Each implementation will support a different kind of Identification algorithm and user interaction.

The way of specifying the observed character values is in in the child classes.

ISSUE: Identification interface should support history of user commands, probably through the Command Design Pattern (see Design patterns by E. Gamma et al., the "gang of four") .

@author Jean-Marc Vanel 
<a href="mailto:jmvanel@free.fr"> jmvanel@free.fr </a> , 
<a href="http://wwbota.free.fr/"> Worldwide Botanical Knowledge Base </a>
*/

public interface Identification {
 /** Get the list of items satisfying the Criteria entered so far.
 @return a list of {@link Description } objects */
 List getMatches();
 /** Which are the best characters to use now?
 @return a list of {@link Character } objects, which are the best separating characters to use at this point in the identification process . */
 List getAdvices();
 /** Get and set the list of Taxon (items) to be recognized, together with  the characters taken in account. */
 Taxonomy getTaxonomy();
 void setTaxonomy( Taxonomy t);
 /** Get and set the list of Criteria, that is, the observed characters. */
 Description getDescription();
 void setDescription( Description t);
}
