package org.wwbota.identification;
import java.util.*;
import java.io.*;

/** A Character with Real Number values, specified by an interval .  */
public class RealNumberCharacter implements Character {
 public RealNumberCharacter(String name, String unit){
  name_=name;
  unit_=unit;
 }

 /** A factory for a Criterion of the type associated to the Character */
 public Criterion makeCriterion(String spec){
  return new IntervalCriterion(this, spec);
 }

 class IntervalCriterion implements Criterion{
  IntervalCriterion(RealNumberCharacter ch, String spec){
   character_ = ch;
   try {
   StreamTokenizer t = new StreamTokenizer(new StringReader(spec));
   int c = t.nextToken();  // read a token
   /*if (c != t.TT_NUMBER )     // check the token type
    throw new IOException("RealNumberCharacter: Missing Real Number for interval");
   */
   lower_ = t.nval; 
   upper_ = lower_;
   c = t.nextToken();
   if (c == t.TT_EOF ) {
    return;
   }
   else { // Now we either need a space or a comma
    if ( c == ',' ) {
     c = t.nextToken();
    }
    if (c == t.TT_NUMBER )
     upper_ = t.nval; 
   }
   }
   catch (IOException e){
    System.err.println( "IOException: IntervalCriterion(RealNumberCharacter ch, String spec): "+e);
   }
  }
  public double isCompatibleWith(Criterion reference){
   IntervalCriterion ref = (IntervalCriterion)reference;
   if ( ref.lower_ <= lower_ && upper_ <= ref.upper_ )
    return 1.;
   else
    return 0.;
  }
  public Character getCharacter(){
   return character_;
  }
  public String toString() {
   return "["+lower_+", "+upper_+"]";
  }
  private RealNumberCharacter character_;
  private double lower_, upper_;
 } // class IntervalCriterion

 public Class getType() {return Double.class; }
 public String toString() {
  return name_ + " (" + unit_+")";
 }
 public String getName() {return name_;}
 public String getURI (){return "http://wwbota.free.fr/Identification/RealNumberCharacter";}

 private String name_;
 private String unit_;
}
