package org.wwbota.identification;
import java.util.*;

/** Computer-aided interactive determination of a biological specimen, believed to belong to one of the taxa of the current {@link Taxonomy Taxonomy} .
This implementation allows any type of {@link Character Character}. 

It checks that every Criterion probability is stricly superior to a threshold, or the Character is undefined either in the observations or in the database. 
The threshold value is zero, which implies a cautious behaviour: never eliminate a Taxon if it has a chance to match.

@author Jean-Marc Vanel 
<a href="mailto:jmvanel@free.fr"> jmvanel@free.fr </a> , 
<a href="http://wwbota.free.fr/"> Worldwide Botanical Knowledge Base </a>
 */
public class SimpleIdentification implements Identification {
 public final double threshold = 0.;
 public SimpleIdentification(Taxonomy t) {
  taxonomy_ = t;
  observed_ = new StandardDescription(null);
 }
 /** returns a list of Description */
 public List getMatches() {
  List result = new ArrayList();
  Map descriptions = taxonomy_.getDescriptions();
  for (Iterator i = descriptions.keySet().iterator(); i.hasNext(); ) {
    // By reference we mean here a reference Description, with which the observed specimen is compared, not a computer reference -;)
    Description reference = (Description)descriptions.get(i.next());
    if ( observed_.isCompatibleWith( reference ) > threshold ) {
     result.add(reference);
    }
  }
  return result;
 }
 /** returns a list of Character, supposed the best to achieve determination.
     NOT IMPLEMENTED: returns all Characters */
 public List getAdvices() {
  //?????? Algorithm ? see R. J. Pankhurst p. 192 Practical Taxonomic Computing
  return new ArrayList(taxonomy_.getCharacters().values() );
 }

 public Taxonomy getTaxonomy(){return taxonomy_;}
 public void setTaxonomy( Taxonomy t){ taxonomy_=t;}

 public void addCriterion( Character c, String spec) throws StateNotAllowedException{
  Criterion crit = c.makeCriterion(spec);
  observed_.addCriterion(crit);
 }

 public String toString() {
  return "TAXONOMY\n" + taxonomy_.toString() + "\nOBSERVATIONS\n" + observed_.toString();
 }

 public Description getDescription(){
  return observed_;
 }
 public void setDescription( Description t){
  observed_ = t;
 }

 private Taxonomy taxonomy_;
 private Description observed_; // accumulates criteria given by user
}
