package org.wwbota.identification;
import java.util.*;

/**
  This is a generic implementation, since it gets the information by iterating on all criteria, and calling the isCompatibleWith() function on each. This is a simple implementation, since it just takes the minimum of probabilities for all characters.

It can be expanded to have a more sophisticated or optimized implementation.
 */
public class StandardDescription implements Description {
 public StandardDescription(Taxon taxon){taxon_=taxon;} 
 public Taxon getTaxon(){return taxon_;}

 // peut-etre mieux d'avoir aussi (ou seulement ?) une Map des Criterion 

 /** @return a Map of {@link Criterion} objects; the key is the corresponding {@link Character} */
 public Map getMap() {
  return features_;
 }

 public void addCriterion(Criterion c){
  features_.put(c.getCharacter(), c);
 }
 /** Accessor to Criterion from Character.
  This is a helper function, since the getMap() accessor gives the same information, plus the ability to iterate on all criteria */
 public Criterion getCriterion(Character c){
  return (Criterion)features_.get(c);
 }


 /** @return the probability that this Description corresponds to the given reference.
 Simply takes the minimum of probabilities for all characters. */
 public double isCompatibleWith(Description reference) {
  Map m = getMap();
  Map mref = reference.getMap();
  double result = 1.0;
  // Each Criterion of this Description must be compatible with the reference
  for (Iterator i = m.keySet().iterator(); i.hasNext(); ) {
   Character c = (Character) i.next();
   Criterion referenceCriterion = reference.getCriterion(c); 
   Criterion thisCriterion = this.getCriterion(c); 
   if ( thisCriterion != null && referenceCriterion != null ) {
    double thisResult = 
     thisCriterion.isCompatibleWith (referenceCriterion); 
    if ( thisResult < result ) {
     result = thisResult;
    }
   }
  }
  return result;
 }

 public String toString() {
  Map m = getMap();
  StringBuffer s = new StringBuffer("");
  for (Iterator i = m.keySet().iterator(); i.hasNext(); ) {
   Character c = (Character) i.next();
   s.append(c.getName()+ ": "+ getCriterion(c)+ ", ");
  }
  String taxon = ( taxon_ != null ? taxon_.toString()+ ": " : "" );
  return taxon  + s.toString();
 }

 private Taxon taxon_;
 private Map features_ = new HashMap();
}
